#!/usr/bin/perl
#
# Extract all URLs from the bbl files and wget them into
# the Literature subdirectory
#
push(@ARGV, './') unless @ARGV;

    # Print HTML
    print <<"ENDOFHEADER";
<html>
<head>
<title>Literature</title>
</head>
<body>
ENDOFHEADER

my $GetSiteCommand = 
"wget -nv --recursive -l 4 --no-parent --page-requisites  --convert-links --no-clobber --html-extension ";
my @ignorelist = ('www.abelard.org', 'homepages.inf.ed.ac.uk/s0450736',
                   'www.gutenberg.org', 'www.cs.colorado.edu', 'www.fon.hum.uva.nl/Service/');
my @cURLlist = ('arxiv.org/pdf/');
foreach my $directory (@ARGV)
{
    # Create Literature subdirectory
    $directory .= '/' unless $directory =~ m!/$!;
    
    print "<h1 align='CENTER'>$directory</h1>\n";

    
    # Get all bbl files
    my @filelist = glob("{$directory}*LOTwinterschool2006.bbl");
    foreach $file (@filelist)
    {
        $file =~ m!(^|/)([^/]+)\.[^\.]+$!;
        my $basename = ($2 || $file);
        my $targetdir = "${directory}Literature/$basename";
        `mkdir -pv $targetdir`;
        
        print "<h2 align='CENTER'>$basename</h2>\n";
        print "<ul>\n";
        
        # open bbl file and read all URLs
        my @URLlines = ();
        my $currentURL = "";
        my $Title = "";
        open(BBLFILE, $file) || die "$file: $!\n";
        while(<BBLFILE>)    # Watch out for misconfigured files
        {
            chomp;
            # Ignore non URL lines
            if($_ =~ /\\bibitem\s*\[/i)
            {
                $Title = $';
                $Title =~ /\]/;
                $Title = $`;
            };
            next if ($_ !~ /\\url\{/) && (length($currentURL) == 0);
            $currentURL .= $_;
            $currentURL =~ s/\%\s*$//g;
            $currentURL =~ s/\s+$//g;
            
            # get the URL if it is complete
            my $LocalLink = "";
            if($currentURL =~  /\\url\{([^\}]+)\}/)
            {
                $currentURL = $1;
                $currentURL =~ s/\\//g;
                push(@URLlines, $currentURL);
                
	            my $ignoreOR = join('|', @ignorelist);
	        
                $LocalLink = "$basename/$currentURL" unless $currentURL !~ /\.(pdf|ps)$/i && $currentURL =~ m!($ignoreOR)!;
            };
            
            # Adapt the local link
            $LocalLink =~ s!(http|ftp)\://!!isg;
            $LocalLink =~ s!/$!/index.html!isg;
            # Arxiv.org does not accept wget requests, use curl
            my $cURL_OR = join('|', @cURLlist);
            if(grep(m!($cURL_OR)!, $LocalLink))
            {
                $LocalLink .= ".pdf" unless $LocalLink =~ /\.pdf$/;
	    };
            
            $Title = $LocalLink unless $Title;
            $Title =~ s!^$basename/!!g;
            print "<li><a href='$LocalLink'>$Title</a></li>\n" if $LocalLink;
        };
        close(BBLFILE);
        foreach my $url (@URLlines)
        {
	        my $ignoreOR = join('|', @ignorelist);
            next if $url !~ /\.(pdf|ps)$/i && $url =~ m!($ignoreOR)!;
	        
	    my $LocalLink = $url;
            $LocalLink =~ s!(http|ftp)\://!!isg;
            $LocalLink =~ s!/$!/index.html!isg;
	    $LocalLink = "${targetdir}/$LocalLink";
            # Arxiv.org does not accept wget requests, use curl
            my $cURL_OR = join('|', @cURLlist);
            if(grep(m!($cURL_OR)!, $LocalLink))
            {
                $LocalLink .= ".pdf" unless $LocalLink =~ /\.pdf$/;
	    };    
	    
            # Arxiv.org does not accept wget requests, use curl
            my $cURL_OR = join('|', @cURLlist);
            if($url =~ /\.(ps|pdf)$/i || grep(m!($cURL_OR)!, $url))
            {
                my $localdir = $LocalLink;
                $LocalLink .= ".pdf" unless $LocalLink =~ /\.pdf$/;
                $localdir =~ s!/[^/]*$!!isg;
                `mkdir -p $localdir`;
                `curl $url > $LocalLink` unless -s "$LocalLink";
            };
            
            print STDERR "$url\n";
            `$GetSiteCommand --directory-prefix=${targetdir}/ $url`;
        };
        
        print "</ul>\n";
    };
};

print "</body>\n</html>\n";

